So, say there is a remote commandable, w, and the client has a proxy
to w, called wp.
The client wants to do something like
wp.addActionListener(L).

L must be a java.awt.event.ActionListener that can be sent to w, so it
should be a Commandable too.


;;; Here's a functioning spec of some ideas that came out of talking
;;; with Ben.

;;; I have a small library of relational operations (like SQL with
;;; functional syntax) that i use to grovel over data.  Ignore the
;;; syntactic details.  The idea is that you can construct a fairly
;;; powerful language out of a few constructs presented in a language
;;; neutral (though maybe functional or relatlional) way, possibly in
;;; a simple GUI

;;; The language can perform optimizations on itself, which this code
;;; does not do.  I have a paper describing these optimizations.

;;; You can C-c C-e or C-M-X to evaluate each expression in silk as
;;; you read this.

;;; Here's the code i use:
(load "elf/basic.scm")
(load "elf/sort.silk")
(load "elf/jdbc.scm")

;;; This is how i got some data to play with
(begin
  (register-driver 'oracle)

  (define con (DriverManager.getConnection 
	       "jdbc:oracle:thin:@stout:1521:dart" "ntranscom" "ntranscom"))

;;; Trim and intern string from database.
  (define-method (datum-value (value String)) (.intern (.trim value)))


  (define data
    (cdr
     (collect-query
      con
      "select 
 GEOLOC_CODE,
 LOCATION_NAME,
 INSTALLATION_TYPE_CODE,
 COUNTRY_STATE_CODE,
 COUNTRY_STATE_LONG_NAME,
 LATITUDE,
 LONGITUDE,
 LOGISTIC_PLANNING_CODE,
 CIVIL_AVIATION_CODE
from geoloc where country_state_long_name like 'AFG%'")))
  )

(define data
  '(
    ;; #("GEOLOC_CODE" "LOCATION_NAME" "INSTALLATION_TYPE_CODE" "COUNTRY_STATE_CODE" "COUNTRY_STATE_LONG_NAME" "LATITUDE" "LONGITUDE" "LOGISTIC_PLANNING_CODE" "CIVIL_AVIATION_CODE")
    #("AJAU" "AJRESTAN SOUTH" "APT" "AF" "AFGHANISTAN" 33.4917 67.1417 "54" #null)
    #("ATUH" "BAGRAM" "MAP" "AF" "AFGHANISTAN" 34.95 69.2672 "54" #null)
    #("BABH" "BAND E SARDEH DAM" "APT" "AF" "AFGHANISTAN" 33.3214 68.6356 "54" #null)
    #("CAHG" "BOST" "APT" "AF" "AFGHANISTAN" 31.5622 64.3619 "54" "OABT")
    #("DEAS" "ZARANJ" "APT" "AF" "AFGHANISTAN" 30.9731 61.8675 "54" #null)
    #("DHJY" "CHAKHCHARAN" "APT" "AF" "AFGHANISTAN" 34.5278 65.2694 "54" "OACC")
    #("FDHW" "DEH DADI" "MAP" "AF" "AFGHANISTAN" 36.6433 67.03 "54" #null)
    #("GAAR" "GARDEZ" "APT" "AF" "AFGHANISTAN" 33.6125 69.2075 "54" "OAGZ")
    #("GAAS" "GARDEZ OLD" "APT" "AF" "AFGHANISTAN" 33.6225 69.2286 "54" #null)
    #("GLCY" "FAIZABAD" "APT" "AF" "AFGHANISTAN" 37.1164 70.5175 "54" "OAFZ")
    #("KHAG" "KHOST" "APT" "AF" "AFGHANISTAN" 33.3306 69.95 "54" "OAKS")
    #("KLZT" "HERAT" "APT" "AF" "AFGHANISTAN" 34.2069 62.2267 "54" "OAHR")
    #("LSLR" "JALALABAD" "APT" "AF" "AFGHANISTAN" 34.4014 70.4994 "54" "OAJL")
    #("LXDS" "KABUL" "AIN" "AF" "AFGHANISTAN" 34.5 69.1667 "54" #null)
    #("LXDW" "KABUL INTL" "JAP" "AF" "AFGHANISTAN" 34.5642 69.2094 "54" "OAKB")
    #("LXDY" "KABUL" "ADM" "AF" "AFGHANISTAN" 34.5 69.1667 "54" #null)
    #("LXEC" "KABUL" "CTY" "AF" "AFGHANISTAN" 34.5 69.1667 "54" #null)
    #("LYAV" "KANDAHAR" "JAP" "AF" "AFGHANISTAN" 31.6222 65.7653 "54" "OAKN")
    #("MAAQ" "MAIMANA" "APT" "AF" "AFGHANISTAN" 35.9281 64.7564 "54" "OAMN")
    #("MEC9" "KHVEJ GHAR" "APT" "AF" "AFGHANISTAN" 37.0633 69.4819 "54" "OAKG")
    #("MECH" "KHVEJ GHAR NORTH" "AFD" "AF" "AFGHANISTAN" 37.065 69.4928 "54" #null)
    #("MLRQ" "KUNDUZ" "APT" "AF" "AFGHANISTAN" 36.6583 68.9056 "54" "OAUZ")
    #("MUAR" "MUKUR" "APT" "AF" "AFGHANISTAN" 32.8647 67.8311 "54" "OAMK")
    #("PQBR" "MAZAR I SHARIF" "APT" "AF" "AFGHANISTAN" 36.7042 67.2083 "54" "OAMS")
    #("SADR" "SARHAWDZA" "APT" "AF" "AFGHANISTAN" 33.1556 68.8194 "54" #null)
    #("UNFD" "UNKN AFGHANISTAN" "RPA" "AF" "AFGHANISTAN" 35 65 "54" #null)
    #("VPTY" "SHINDAND" "MAP" "AF" "AFGHANISTAN" 33.3903 62.2556 "54" "OASD")
    ))

(define (curry2 op arg2)
  ;; Predicates come from curried relations.
  (lambda (arg1) (op arg1 arg2)))

;;; So do positional accessors.
;;; This code generates the following definitions:
(define ds
  (for-each
   (let ((i -1))
     (lambda (n) (print `(define ,(string->symbol n)
			   (curry2 vector-ref ,(set! i (+ i 1)))))))
  '("geoloc_code" "location_name" "installation_type_code" "country_state_code" "country_state_long_name" "latitude" "longitude" "logistic_planning_code" "civil_aviation_code")))

(define geoloc_code (curry2 vector-ref 0))
(define location_name (curry2 vector-ref 1))
(define installation_type_code (curry2 vector-ref 2))
(define country_state_code (curry2 vector-ref 3))
(define country_state_long_name (curry2 vector-ref 4))
(define latitude (curry2 vector-ref 5))
(define longitude (curry2 vector-ref 6))
(define logistic_planning_code (curry2 vector-ref 7))
(define civil_aviation_code (curry2 vector-ref 8))

;;; Some comparison constructors:
(define (binop op) (lambda (arg2) (curry2 op arg2)))
(define >: (binop >))
(define >=: (binop >=))
(define <: (binop <))
(define <=: (binop <=))
(define ==: (binop equal?))

;;; Predicate constructors
(define (not: f) (lambda (x) (not (f x))))
(define (and: f g) (lambda (x) (and (f x) (g x))))
(define (or: f g) (lambda (x) (or (f x) (g x))))

(define (filter p)
  ;; Filter sequence xs based on predicate, p.
  (lambda (xs)
    (reverse
     (foldL xs (lambda (x so-far)
		 (if (p x) (cons x so-far)
		     so-far)) '()))))

(define null: (==: #null))
(define notNull: (not: null:))

(define (compose g f) 
  ;; (compose) return a (procedure x) that pipes the result of (f x)
  ;; into (g).
  (lambda (x) (g (f x))))


;;; Here's some sorting code
(define (comparator compare access)
  (elf.SchemeComparator.
   (lambda (a b)
     (let ((a (access a))
	   (b (access b)))
       (if (compare a b) -1
	   (if (compare b a) 1
	       0))))))

(define-method (sort (xs silk.Pair) comparator)
  (let ((it (list->vector xs)))
    (sort it comparator)
    (vector->list it)))

(define-method (sort (xs Object[]) comparator)
  (Arrays.sort xs comparator))

(define (group-by what xs)
  (let ((table (java.util.Hashtable.)))
    (iterate xs
	     (lambda (x)
	       (let* ((key (what x))
		      (items (.get table key)))
		 (.put table key (cons x (if (eq? items #null) '() items))))))
    (.values table)))

(define (unique p)
  (lambda (xs) (p (call/cc (lambda (return)
			     (iterate xs (lambda (x) (return x))))))))

(define (extract . accessors)
  (lambda (x) (map (lambda (a) (a x)) accessors)))

(define (project . as)
  (lambda (xs) (map* (apply extract as) xs)))

(define (count f)
  (lambda (xs)
    (foldL (map* f xs) (lambda (x so-far) (+ x so-far)) 0)))

(define (show f)
  (lambda (xs) (for-each print (f xs))))

(define (show xs) (for-each* print xs))

;;; Here are sample queries:
;;; select * from data where civil_avation_code is not null;
(show ((filter (compose notNull: civil_aviation_code)) data))
#("CAHG" "BOST" "APT" "AF" "AFGHANISTAN" 31.5622 64.3619 "54" "OABT")
#("DHJY" "CHAKHCHARAN" "APT" "AF" "AFGHANISTAN" 34.5278 65.2694 "54" "OACC")
#("GAAR" "GARDEZ" "APT" "AF" "AFGHANISTAN" 33.6125 69.2075 "54" "OAGZ")
#("GLCY" "FAIZABAD" "APT" "AF" "AFGHANISTAN" 37.1164 70.5175 "54" "OAFZ")
#("KHAG" "KHOST" "APT" "AF" "AFGHANISTAN" 33.3306 69.95 "54" "OAKS")
#("KLZT" "HERAT" "APT" "AF" "AFGHANISTAN" 34.2069 62.2267 "54" "OAHR")
#("LSLR" "JALALABAD" "APT" "AF" "AFGHANISTAN" 34.4014 70.4994 "54" "OAJL")
#("LXDW" "KABUL INTL" "JAP" "AF" "AFGHANISTAN" 34.5642 69.2094 "54" "OAKB")
#("LYAV" "KANDAHAR" "JAP" "AF" "AFGHANISTAN" 31.6222 65.7653 "54" "OAKN")
#("MAAQ" "MAIMANA" "APT" "AF" "AFGHANISTAN" 35.9281 64.7564 "54" "OAMN")
#("MEC9" "KHVEJ GHAR" "APT" "AF" "AFGHANISTAN" 37.0633 69.4819 "54" "OAKG")
#("MLRQ" "KUNDUZ" "APT" "AF" "AFGHANISTAN" 36.6583 68.9056 "54" "OAUZ")
#("MUAR" "MUKUR" "APT" "AF" "AFGHANISTAN" 32.8647 67.8311 "54" "OAMK")
#("PQBR" "MAZAR I SHARIF" "APT" "AF" "AFGHANISTAN" 36.7042 67.2083 "54" "OAMS")
#("VPTY" "SHINDAND" "MAP" "AF" "AFGHANISTAN" 33.3903 62.2556 "54" "OASD")


;;; select civil_aviation_code, location_name from data where
;;; civil_avation_code is not null;
(show ((project civil_aviation_code location_name)
       ((filter (compose notNull: civil_aviation_code))
	    data)))

("OABT" "BOST")
("OACC" "CHAKHCHARAN")
("OAGZ" "GARDEZ")
("OAFZ" "FAIZABAD")
("OAKS" "KHOST")
("OAHR" "HERAT")
("OAJL" "JALALABAD")
("OAKB" "KABUL INTL")
("OAKN" "KANDAHAR")
("OAMN" "MAIMANA")
("OAKG" "KHVEJ GHAR")
("OAUZ" "KUNDUZ")
("OAMK" "MUKUR")
("OAMS" "MAZAR I SHARIF")
("OASD" "SHINDAND")

;;; select the unique pairs of installation_type_code and
;;; logistic_palnning_code (not sure how to say this in sql).
(show
 ((project (unique installation_type_code)
	   (unique logistic_planning_code))
       (group-by (extract installation_type_code logistic_planning_code)
		 data)))

("APT" "54")
("AIN" "54")
("MAP" "54")
("JAP" "54")
("ADM" "54")
("RPA" "54")
("CTY" "54")
("AFD" "54")

(define (count: f) (lambda (xs) (length (f xs))))

;;; For each unique pairs of installation_type_code and
;;; logistic_palnning_code show the number of locations with that pair
;;; and the number that also have ICAO codes.

(show
 ((project (unique installation_type_code)
	   (unique logistic_planning_code)
	   length
	   (count: (filter (compose notNull: civil_aviation_code))))
  (group-by (extract installation_type_code logistic_planning_code)
	    data)))

("APT" "54" 17 12)
("AIN" "54" 1 0)
("MAP" "54" 3 1)
("JAP" "54" 2 2)
("ADM" "54" 1 0)
("RPA" "54" 1 0)
("CTY" "54" 1 0)
("AFD" "54" 1 0)

;;; select location_name from data where latitude > 34.0;
(show
 ((project location_name)
  ((filter (compose (>: 34.0) latitude))
   data)))

("BAGRAM")
("CHAKHCHARAN")
("DEH DADI")
("FAIZABAD")
("HERAT")
("JALALABAD")
("KABUL")
("KABUL INTL")
("KABUL")
("KABUL")
("MAIMANA")
("KHVEJ GHAR")
("KHVEJ GHAR NORTH")
("KUNDUZ")
("MAZAR I SHARIF")
("UNKN AFGHANISTAN")
