;;; These work with any permission.
(define (find-field class name)
  (find (lambda (f) (.equals (.getName f) name)) ; Memoizable.
	(all-fields class)))

(define (field object name)
  (.get (find-field (.getClass object) (.toString name)) object))

(define (field-set! object name value)
  (.set (find-field (.getClass object) (.toString name)) object value))

(define (static-field class name)
  (.get (find-field class (.toString name)) class))

(define (static-field-set! class name value)
  (.set (find-field class (.toString name)) class value))

(define (isNull x) (eq? x #null))

(define (dump x port so-far)
  (if (isNull x)
      (display "#null" port)
      (let ((it (.get so-far x)))
	(if (not (isNull it)) (display `([id] ,it))
	    (begin
	      (.put so-far x (.size so-far))
	      (dumpObject x port so-far)))))
  so-far)

(define-method (dumpObject (x Boolean) port so-far) (write x port))
(define-method (dumpObject (x Number) port so-far) (write x port))
(define-method (dumpObject (x String) port so-far) (write x port))
(define-method (dumpObject (x Symbol) port so-far)
  (display #\' port)
  (write x port))

(define-method (dumpObject (x Object) port so-far)
  (newline port)
  (display (string-append
	    "(load " (- (.size so-far) 1) " " (.getName (.getClass x)) " ")
	   port)
  (for-each (lambda (f) (dumpField f (.get f x) port so-far))
	    (objectFields x))
  (display ") " port))

(define-method (objectFields (x Object))
  (filter-in (lambda (f)
	       (let ((ms (.getModifiers f)))
		 (and (not (Modifier.isStatic ms))
		      ;; (not (Modifier.isTransient ms))
		      )))
	     (all-fields (.getClass x))))

(define (dumpField f v port so-far)
  (display " " port)
  (write (.getName f) port)
  (display " " port)
  (dump v port so-far))

(define (fields-named x . names)
  (filter-in (lambda (f) (member (.getName f) names))
	     (all-fields (.getClass x))))

(define-method (dumpObject (x silk.Pair) port so-far)
  (if (null? x) (display '() port)
      (begin
	(display "(cons " port)
	(dump (car x) port so-far)
	(display " " port)
	(dump (cdr x) port so-far)
	(display ")" port))))

(define-method (dumpObject (x Object[]) port so-far)
  (newline port)
  (display
   (string-append "(loadVector " (- (.size so-far) 1) " "
		  (.getName (.getComponentType (.getClass x))))
   port)
  (for-each* (lambda (i)
	       (display " " port)
	       (dump i port so-far))
	     x)
  (display ")" port))

(let* ((data (Vector.))
       ([id] (lambda (
(load 0 java.util.Vector
      "capacityIncrement" 0
      "elementCount" 1
      "elementData" (loadVector 3 java.lang.Object
				(cons 'a (cons 'b ()))
				#null
				#null
				#null
				#null
				#null
				#null
				#null
				#null
				#null)
      "modCount" ([id] 2))