(import "java.awt.BorderLayout")
(import "java.awt.Color")
(import "java.awt.Dimension")
(import "java.awt.FlowLayout")
(import "java.awt.GridLayout")
(import "java.lang.Double")
(import "javax.swing.BorderFactory")
(import "javax.swing.Box")
(import "javax.swing.BoxLayout")
(import "javax.swing.JButton")
(import "javax.swing.JCheckBox")
(import "javax.swing.JComboBox")
(import "javax.swing.JFrame")
(import "javax.swing.JLabel")
(import "javax.swing.JList")
(import "javax.swing.JPanel")
(import "javax.swing.JTextField")
(import "javax.swing.UIManager")

(UIManager.setLookAndFeel (UIManager.getSystemLookAndFeelClassName))

(define (column-panel . items)
  (let ((p (JPanel.)))
    (.setLayout p (GridLayout. 0 1))
    (for-each 
     (lambda (item) (.add p item))
     items)
    p))

(define (row-panel . items)
  (let ((p (JPanel.)))
    (.setLayout p (GridLayout. 1 0))
    (for-each 
     (lambda (item) (.add p item))
     items)
    p))

(define (frame-it name panel)
  (let ((frame (JFrame. name)))
    (.setContentPane frame panel)
    (.pack frame)
    (.show frame)
    frame))

(define (borderPanel c n s e w)
  (define (maybe-add p x how)
    (if (and x (not (null? x))) (.add p x how)))
  (let ((panel (JPanel.(BorderLayout.))))
    (maybe-add panel c BorderLayout.CENTER$)
    (maybe-add panel n BorderLayout.NORTH$)
    (maybe-add panel s BorderLayout.SOUTH$)
    (maybe-add panel e BorderLayout.EAST$)
    (maybe-add panel w BorderLayout.WEST$)
    panel))

(define (north-south n s) (borderPanel '() n s '() '()))

(define (flowPanel how . items)
  (let ((result (JPanel. (FlowLayout. how))))
    (for-each (lambda (x) (.add result x)) items)
    result))

(define (Panel . items)
  (let ((result (JPanel.)))
    (for-each (lambda (x) (.add result x)) items)
    result))


(define (blackLabel title orientation)
  ;; A JLabel with black text, rather than blue.  Why is the default blue?
  (let ((result (JLabel. title orientation)))
    (.setForeground result Color.black$)
    result))

(define (verticalTextField label size)
  (north-south (JLabel. label SwingConstants.CENTER$) (JTextField. size)))

(define (fix-height x) 
  ;; Wrapping something in the north or south of a BorderLayout fixes
  ;; its height.
  (borderPanel '() x '() '() '()))

(define (fix-width x)
  ;; Wrapping something in the east or west of a BorderLayout fixes
  ;; its width.
  (borderPanel '() '() '() x '()))

(define (horizontalTextField label size)
  (fix-width				; ?
   (borderPanel
    '() '() '()
    (JTextField. size)
    (blackLabel label SwingConstants.RIGHT$))))

(define (mission-time-panel title)
  (borderPanel
   ()
   (blackLabel title SwingConstants.CENTER$) ; N
   (JCheckBox. "Hard" #f)		; S
   (verticalTextField "Purpose" 8)
   (verticalTextField "Time" 8)
   ))

'(frame-it "mission" (mission-time-panel "Departure"))

(define (HBox . items)
  (let ((result (Box.createHorizontalBox)))
    (for-each (lambda (x) (.add result x)) items)
    result))

(define (VBox . items)
  (let ((result (Box.createVerticalBox)))
    (for-each (lambda (x) (.add result x)) items)
    result))

(define (simple-field label size)
  (borderPanel 
   '() 
   (blackLabel " " SwingConstants.CENTER$)
   (let ((result (blackLabel " " SwingConstants.CENTER$)))
     (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
     result)
   (verticalTextField label size) 
   '() 
   ))

(define (try-6)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (frame-it 
   "Try 6" 
   (fix-height
    (HBox 
     (mission-time-panel "Arrival")
     (simple-field "ICAO" 4)
     (mission-time-panel "Departure")
     (simple-field "Ground Time" 11)
     (JCheckBox. "As Required" #f)
     ))))

(define (try-7)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (frame-it 
   "Try 6" 
   (VBox 
    (fix-height 
     (HBox ;; flowPanel FlowLayout.CENTER$
		(horizontalTextField "Leg ID:" 5)
		(horizontalTextField "MSn #:" 10)
		(horizontalTextField "Priority:" 4)
		(horizontalTextField "Crew Type:" 7)
		))
    (fix-height
     (HBox 
      (mission-time-panel "Arrival")
      (simple-field "ICAO" 4)
      (mission-time-panel "Departure")
      (simple-field "Ground Time" 11)
      (JCheckBox. "As Required" #f)
      )))))

(define (try-8)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (frame-it 
   "Try 8" 
   (fix-height
    (VBox
     (HBox
      (horizontalTextField "Leg ID: " 5)
      (horizontalTextField "MSn #: " 10)
      (horizontalTextField "Priority: " 4)
      (horizontalTextField "Crew Type: " 7)
      )
     (HBox 
      (mission-time-panel "Arrival")
      (simple-field "ICAO" 4)
      (mission-time-panel "Departure")
      (simple-field "Ground Time" 11)
      (JCheckBox. "As Required" #f)
      )))))

; (define (titledBorder panel)
;  (setBorder panel 

(define (try-9)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (let ((panel 
	 (fix-height
	  (VBox
	   (HBox
	    (horizontalTextField "Leg ID: " 5)
	    (horizontalTextField "MSn #: " 10)
	    (horizontalTextField "Priority: " 4)
	    (horizontalTextField "Crew Type: " 7)
	    )
	   (HBox 
	    (mission-time-panel "Arrival")
	    (simple-field "ICAO" 4)
	    (mission-time-panel "Departure")
	    (simple-field "Ground Time" 11)
	    (JCheckBox. "As Required" #f)
	    )))))
    (.setBorder panel (BorderFactory.createTitledBorder "Location"))
    (frame-it 
     "Try 9" 
     panel)))

(define (try-10)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (let ((panel 
	 (fix-height
	  (VBox
	   (HBox
	    (horizontalTextField "Leg ID: " 5)
	    (horizontalTextField "MSn #: " 10)
	    (horizontalTextField "Priority: " 4)
	    (horizontalTextField "Crew Type: " 7)
	    )
	   (HBox 
	    (mission-time-panel "Arrival")
	    (simple-field "ICAO" 4)
	    (mission-time-panel "Departure")
	    (simple-field "Ground Time" 11)
	    (JCheckBox. "As Required" #f)
	    )))))
    (.setBorder panel (BorderFactory.createTitledBorder "Location"))
    (frame-it 
     "Try 10" 
     panel)))

(define (try-11)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (let ((panel 
	 (fix-height
	  (VBox
	   (HBox
	    (horizontalTextField "Leg ID: " 5)
	    (horizontalTextField "MSn #: " 10)
	    (horizontalTextField "Priority: " 4)
	    (horizontalTextField "Crew Type: " 7)
	    )
	   (HBox 
	    (mission-time-panel "Arrival")
	    (simple-field "ICAO" 4)
	    (mission-time-panel "Departure")
	    (simple-field "Ground Time" 11)
	    (JCheckBox. "As Required" #f)
	    )))))
    (.setBorder panel (BorderFactory.createTitledBorder "Location"))
    (frame-it 
     "Try 11" 
     (north-south
      panel
      (flowPanel FlowLayout.LEFT$
		 (horizontalTextField  "Remarks: " 
				       40
				       ))))))

(define (try-11)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (let ((panel 
	 (fix-height
	  (VBox
	   (HBox
	    (horizontalTextField "Leg ID: " 5)
	    (horizontalTextField "MSn #: " 10)
	    (horizontalTextField "Priority: " 4)
	    (horizontalTextField "Crew Type: " 7)
	    )
	   (HBox 
	    (mission-time-panel "Arrival")
	    (simple-field "ICAO" 4)
	    (mission-time-panel "Departure")
	    (simple-field "Ground Time" 11)
	    (JCheckBox. "As Required" #f)
	    )))))
    (.setBorder panel (BorderFactory.createTitledBorder "Location"))
    (frame-it 
     "Try 11" 
     (north-south
      panel
      (flowPanel FlowLayout.LEFT$
		 (horizontalTextField  "Remarks: " 
				       40
				       ))))))

(define (try-12)
  (define (simple-field label size)
    (borderPanel 
     '() 
     (blackLabel " " SwingConstants.CENTER$)
     (let ((result (blackLabel " " SwingConstants.CENTER$)))
       (.setPreferredSize result (.getPreferredSize (JCheckBox. "Hard" #f)))
       result)
     (verticalTextField label size) 
     '() 
     ))
  (let ((panel 
	 (fix-height
	  (VBox
	   (HBox
	    (horizontalTextField "Leg ID: " 5)
	    (horizontalTextField "MSn #: " 10)
	    (horizontalTextField "Priority: " 4)
	    (horizontalTextField "Crew Type: " 7)
	    )
	   (HBox 
	    (mission-time-panel "Arrival")
	    (Box.createHorizontalStrut 5)
	    (simple-field "ICAO" 4)
	    (Box.createHorizontalStrut 5)
	    (mission-time-panel "Departure")
	    (Box.createHorizontalStrut 5)
	    (simple-field "Ground Time" 11)
	    (JCheckBox. "As Required" #f)
	    (Box.createHorizontalGlue)
	    )))))
    (.setBorder panel (BorderFactory.createTitledBorder "Location"))
    (frame-it 
     "Try 12" 
     (north-south
      panel
      (flowPanel FlowLayout.LEFT$
		 (horizontalTextField  "Remarks: " 40))))))

(define (try-1)
  (frame-it 
   "try-1"
   (fix-height
   (HBox
    (VBox (horizontalTextField "Msn#: " 12)
	  (horizontalTextField "NAF: " 5)
	  (horizontalTextField "Wing: " 12)
	  (horizontalTextField "Sqd: " 12))
    (VBox (horizontalTextField "Alias: " 8)
	  (horizontalTextField "Status: " 8)
	  (horizontalTextField "Msn Priority: " 4)
	  (horizontalTextField "Local Priority: " 4))
    (VBox (horizontalTextField "Msn Type: " 11)
	  (horizontalTextField "Reason: " 11)
	  (horizontalTextField "Cmdr: " 13)
	  (horizontalTextField "Call Sign: " 13))
    (VBox (horizontalTextField "MDS: " 7)
	  (horizontalTextField "Symbol: " 5)
	  (horizontalTextField "OPLAN: " 6)
	  (horizontalTextField "Parking Spot: " 5))
    (VBox (horizontalTextField "Tail #: " 6)
	  (horizontalTextField "Config: " 6)
	  (horizontalTextField "Published: " 2)
	  (horizontalTextField "Close Watch: " 2))
    ))))

(define (try-2)
  (frame-it 
   "try-2"
   (HBox
    (Box.createHorizontalGlue)
    (VBox (horizontalTextField "Msn#: " 12)
	  (horizontalTextField "NAF: " 5)
	  (horizontalTextField "Wing: " 12)
	  (horizontalTextField "Sqd: " 12))
    (VBox (horizontalTextField "Alias: " 8)
	  (horizontalTextField "Status: " 8)
	  (horizontalTextField "Msn Priority: " 4)
	  (horizontalTextField "Local Priority: " 4))
    (VBox (horizontalTextField "Msn Type: " 11)
	  (horizontalTextField "Reason: " 11)
	  (horizontalTextField "Cmdr: " 13)
	  (horizontalTextField "Call Sign: " 13))
    (fix-width
    (VBox (horizontalTextField "MDS: " 7)
	  (horizontalTextField "Symbol: " 5)
	  (horizontalTextField "OPLAN: " 6)
	  (horizontalTextField "Parking Spot: " 5)))
    (VBox (horizontalTextField "Tail #: " 6)
	  (horizontalTextField "Config: " 6)
	  (horizontalTextField "Published: " 2)
	  (horizontalTextField "Close Watch: " 2))
    (Box.createHorizontalGlue)
    )))


(frame-it
 "Fred"
 (borderPanel
  (row-panel
   (with
    (borderPanel
     (JButton. "Center")
     ()
     (flowPanel FlowLayout.RIGHT$ (JButton. "Fail"))
     ()
     ())
    border (javax.swing.BorderFactory.createTitledBorder "COA"))
   (with
    (column-panel
     (JButton. "Top")
     (JButton. "Bottom"))
    border (javax.swing.BorderFactory.createTitledBorder "Right")))
  ()
  (flowPanel FlowLayout.RIGHT$ (JButton. "Lou Mason"))
  ()
  ()))
