(import "elf.Listen")
(import "java.awt.BorderLayout")
(import "java.awt.Dimension")
(import "java.awt.event.ActionEvent")
(import "javax.swing.ButtonGroup")
(import "javax.swing.JCheckBoxMenuItem")
(import "javax.swing.JFrame")
(import "javax.swing.JMenu")
(import "javax.swing.JMenuBar")
(import "javax.swing.JMenuItem")
(import "javax.swing.JRadioButtonMenuItem")
(import "javax.swing.JSplitPane")
(import "javax.swing.JTextArea")

(define true #t)
(define false #f)

(define (make-tagger)
  (let ((table (java.util.Hashtable. 20)))
    (lambda (name . maybe-value)
      (if (null? maybe-value) (.get table name)
	  (begin (.put table name (car maybe-value))
		 (.get table name))))))

(define (method-name prefix name)
  ;; Generate an accessor name.
  (let ((s (symbol->string name)))
    (string->symbol
     (string-append "." prefix (char-upcase (string-ref s 0)) (.substring s 1)))))

(define-macro (with object . args) (expand-with object args))

(define (expand-with object args)
  `(let ((it ,object))
     ,@(expand-with-args args '())
     it))

(define (expand-with-args args so-far)
  (if (null? args) (reverse so-far)
      (let ((head (car args)))
	(cond 
	 ((symbol? head)
	  (expand-with-args (cddr args) (cons (list (method-name "set" head)'it (cadr args)) so-far)))
	 ((pair? head)
	  (expand-with-args (cdr args) (cons head so-far)))
	 (else (error "expand-with-args can't handle" args))))))

(define text "This is a simple text string that is long enought to wrap over a few lines in the simple demo we're about to build.  We'll put tow txt areas side by side in a split screen.")

(define (myTextArea text)
  (with (JTextArea. text)
	lineWrap true
	wrapStyleWord true
	minimumSize (Dimension. 150 150)))

'(define
  frame
  (with (JFrame. "Simple SlitPane Frame")
	(.setSize it 450 200)
	(.add (.getContentPane it) 
	     (JSplitPane. JSplitPane.VERTICAL_SPLIT$
			 (with (myTextArea text)
			       preferredSize (Dimension. 250 200))
			 (myTextArea text))
	     )))

'(setVisible frame true)

;;; Menus: Swing book p 412.
(define (menu name . items)
  (let ((it (JMenu. name)))
    ;; (setMnemonic it (char->integer shortcut))
    (for-each (lambda (item) 
		(if (eq? item 'separator)
		    (.addSeparator it)
		    (.add it item)))
	      items)
    it))

(define print-action 
  (silk.Listener.
   (lambda (e) 
     (print (getActionCommand e))
     )))

(define (menuItem name shortcut action)
  (let ((it (JMenuItem. name (char->integer shortcut))))
    (.addActionListener it (Listener. action))
       it))

(define (menuRadioButton name group action)
  (let ((it (JRadioButtonMenuItem. name)))
    (.add group it)
    (.addActionListener it (Listener. action))
    it))

(define (menuBar . items)
  (let ((it (JMenuBar.)))
    (for-each (lambda (i) (.add it i)) items)
    it))

(define (bar)
  (menuBar
   (menu "File"
	 (menuItem "Save as CSV" #\S print-action)
	 (menuItem "Exit" #\X print-action))

   (menu "Edit"
	 (menuItem "Undo"  #\U print-action)
	 'separator
	 (menuItem "Cut"   #\X print-action)
	 (menuItem "Copy"  #\C print-action)
	 (menuItem "Paste" #\V print-action))

   (let ((bgroup (ButtonGroup.)))
     (menu "Connect"
	   (with (menuItem "Disconnect" #\D print-action)
		 enabled #f)
	   'separator
	   (menuRadioButton "Dart" bgroup)
	   (menuRadioButton "AFORMS" bgroup)
	   (menuRadioButton "C2IPS" bgroup)
	   (menuRadioButton "ULPS" bgroup)))
   (menu "Help"
	 (menuItem "About" #\A print-action))
   ))

(define (db-frame)
  (define (bar)
    (menuBar
     (menu "File"
	   (menuItem "Save as CSV" #\S print-action)
	   (menuItem "Exit" #\X print-action))

     (menu "Edit"
	   (menuItem "Undo"  #\U print-action)
	   'separator
	   (menuItem "Cut"   #\X print-action)
	   (menuItem "Copy"  #\C print-action)
	   (menuItem "Paste" #\V print-action))

     (let ((bgroup (ButtonGroup.)))
       (menu "Connect"
	     (with (menuItem "Disconnect" #\D print-action)
		   enabled #f)
	     'separator
	     (menuRadioButton "Dart" bgroup)
	     (menuRadioButton "AFORMS" bgroup)
	     (menuRadioButton "C2IPS" bgroup)
	     (menuRadioButton "ULPS" bgroup)))
     (menu "Help"
	   (menuItem "About" #\A print-action))
     ))
  (let ((frame (JFrame. "ULPS database")))
    (.setJMenuBar frame (bar))
    (.pack frame)
    (.show frame)
    frame
    ))

