;;; Philip Wadler, Comprehending Monads

;;; Goguen's thesis - first-order languages are sufficient for most purposes.

;;; List monad.

;;; map :: (x -> y) -> (M x -> M y).

(define (MAP f)
  (lambda (xs)
    (if (null? xs) '()
	(cons (f (car xs)) ((MAP f) (cdr xs))))))

(define (MAP f)
  (letrec
      ((self (lambda (xs)
	       (if (null? xs) '()
		   (cons (f (car xs)) (self (cdr xs)))))))
    self))

(define ID (lambda (x) x))

(define (COMPOSE f g) (lambda (x) (f (g x))))

(COMPOSE (lambda (x) (* 2 x)) (lambda (x) (+ 3.5 x)))

(define (ISEQUAL f g) (lambda (xs) (equal? (f xs) (g xs))))

(define (ISEQUAL2 f)
  (assert (f '(1 2 3)))
  (assert (f '()))
  #t)

;;; (i)
(ISEQUAL2 (ISEQUAL (MAP ID) ID))

(assert ((ISEQUAL (MAP ID) ID) '(1 2 3)))
(assert ((ISEQUAL (MAP ID) ID) '()))

(define P1 (lambda (x) (+ x 1)))
(define T2 (lambda (x) (* x 2)))

;;; (ii)
(let ((F (ISEQUAL (MAP (COMPOSE P1 T2)) (COMPOSE (MAP P1) (MAP T2)))))
  (assert (F '(1 2 3)))
  (assert (F '())))

(define M list)

;;; unit :: x -> M x
(define UNIT M)

;;; join :: M(M x) -> M x
(define JOIN (lambda (xs) (apply append  xs)))

(define x (list (list 1 2) (list 3 4) (list 5)))

(assert (equal? (JOIN x) '(1 2 3 4 5)))

(ISEQUAL2 (ISEQUAL (COMPOSE (MAP P1) UNIT) (COMPOSE UNIT P1)))