;;; Given a class, using public introspection, how many other classes can
;;; you find?  Examples:

;;; > (length (reach-control '() (list (class "java.lang.Object"))))
;;; 203
;;; > (length (reach-control '() (list (class "com.bbn.openmap.app.OpenMap"))))
;;; 615
;;; > (length (reach-control '() (list (class "silk.Scheme"))))
;;; 445
;;; > (length (reach-control '() (list (class "javax.swing.JFrame"))))
;;; 494

(define keep-new
  (lambda (x so-far)
    (if (memq x so-far) so-far
	(cons x so-far))))

(define (keep-new-of f)
  (lambda (x so-far) (keep-new (f x) so-far)))

(define keep-value 
  ;; If the field is static final and its type is either Class or Class[]
  ;; Use its value as a source of classes.
  (let ((Class (class 'java.lang.Class)))
    (lambda (field so-far)
      (if (let ((m (.getModifiers field)))
	    (and (Modifier.isStatic m)
		 (Modifier.isFinal m)
		 (Modifier.isPublic m)))
	  (let ((type (.getType field)))
	    (cond
	     ((.isAssignableFrom Class type)
	      (keep-new (.get field (.getDeclaringClass field)) so-far))
	     ((and (.isArray type) 
		   (.isAssignableFrom Class (.getComponentType type)))
	      (foldL (.get field (.getDeclaringClass field)) keep-new so-far))
	     (else so-far)))
	  so-far))))

(define (reach class so-far)
  (define (foldL-keep-new what of so-far)
    (foldL what (lambda (m so-far) (foldL (of m) keep-new so-far)) so-far))
  (if (.isArray class)
      (reach (.getComponentType class) (keep-new class so-far))
      (let* ((so-far (keep-new class so-far))
	     (so-far (let ((super (.getSuperclass class)))
		       (if (not (eq? super #null)) (keep-new super so-far)
			   so-far)))
	     (so-far (foldL (.getInterfaces class) keep-new so-far))
	     ;; KRA 16JAN99: causes access violation.
	     ;; (so-far (foldL keep-new (.getDeclaredClasses class) so-far))
	     (fields (.getDeclaredFields class))
	     (so-far (foldL fields (keep-new-of .getType) so-far))
	     (so-far (foldL fields keep-value so-far))
	     (methods (.getDeclaredMethods class))
	     (so-far (foldL methods (keep-new-of .getReturnType) so-far))
	     (so-far (foldL-keep-new methods .getParameterTypes so-far))
	     (so-far (foldL-keep-new methods .getExceptionTypes so-far))
	     (so-far (foldL-keep-new (.getDeclaredConstructors class)
				     .getParameterTypes so-far)))
    
	so-far)))

(define (reach-control seen so-far)
  (if (null? so-far) seen
      (let ((it (car so-far)))
	(if (memq it seen) (reach-control seen (cdr so-far))
	    (reach-control (cons it seen) (reach it so-far))))))

