(import "java.lang.System")

(define (home) (System.getProperty "user.home"))

(define (pwd) (System.getProperty "user.dir"))

(define (cd . where)
  (let ((goto (if (not (null? where)) (car where) (home))))
    (System.setProperty "user.dir" goto)))

(define (match*0 special? p ps pe t ts te)
  (print (list 'match*0 special? p ps pe t ts te))
  (if (< ps pe)
      (let ((pc (string-ref p ps)))
	(if special?
	    (cond ((eqv? pc #\\) (match*0 #f p (+ ps 1) pe t ts te))
		  ((eqv? pc #\?)
		   (and (< ts te) (match*0 #t p (+ ps 1) pe t (+ ts 1) te)))
		  ((eqv? pc #\*) (or (match*0 #t p (+ ps 1) pe t ts te)
				     (and (< ts te)
					  (match*0 #t p ps pe t (+ ts 1) te))))
		  (else (and (< ts te) (eqv? pc (string-ref t ts))
			     (match*0 #t p (+ ps 1) pe t (+ ts 1) te))))
	    (and (< ts te) (eqv? pc (string-ref t ts))
		 (match*0 #t p (+ ps 1) pe t (+ ts 1) te))))
      (not (< ts te))))

;;; Specialize on special?
(define (match*0t p ps pe t ts te)
  ;; (print (list 'match*0t p ps pe t ts te))
  (if (< ps pe)
      (let ((pc (string-ref p ps)))
	(cond ((eqv? pc #\\) (match*0f p (+ ps 1) pe t ts te))
	      ((eqv? pc #\?)
	       (and (< ts te) (match*0t p (+ ps 1) pe t (+ ts 1) te)))
	      ((eqv? pc #\*) (or (match*0t p (+ ps 1) pe t ts te)
				 (and (< ts te)
				      (match*0t p ps pe t (+ ts 1) te))))
	      (else (and (< ts te) (eqv? pc (string-ref t ts))
			 (match*0t p (+ ps 1) pe t (+ ts 1) te)))))
      (not (< ts te))))

(define (match*0f p ps pe t ts te)
  ;; (print (list 'match*0f p ps pe t ts te))
  (if (< ps pe)
      (let ((pc (string-ref p ps)))
	(and (< ts te) (eqv? pc (string-ref t ts))
			     (match*0t p (+ ps 1) pe t (+ ts 1) te)))
      (not (< ts te))))

(define (compile-pattern pattern)
  (let ((p pattern)
	(pe (string-length pattern)))

(define (compile-pattern0 p ps pe)
  (if (< ps pe)
      (let ((pc (string-ref p ps)))
	(cond ((eqv? pc #\\)
	       (let ((ps (+ ps 1)))
		 (if (< ps pe)
		     `((is ,(string-ref p ps)) ,@(compile-pattern0 p ps pe))
		     `((is ,pc)))))
	      ((eqv? pc #\?) `((any) ,@(compile-pattern0 p (+ ps 1) pe)))
	      ))))

(define (match* pattern target)
  (let ((p pattern)
	(pe (string-length pattern))
	(t target)
	(te (string-length target)))
    (define (match1 special? ps ts)
      (print (list 'match1 special? ps ts))
      (if (< ps pe)
	  (let ((pc (string-ref p ps)))
	    (if special?
		(cond ((eqv? pc #\\) (match1 #f (+ ps 1) ts))
		      ((eqv? pc #\?)
		       (and (< ts te) (match1 #t (+ ps 1) (+ ts 1))))
		      ((eqv? pc #\*) (or (match1 #t (+ ps 1) ts)
					 (and (< ts te)
					      (match1 #t ps (+ ts 1)))))
		      (else (and (< ts te) (eqv? pc (string-ref t ts))
				 (match1 #t (+ ps 1) (+ ts 1)))))
		(and (< ts te) (eqv? pc (string-ref t ts))
		     (match1 #t (+ ps 1) (+ ts 1)))))
	  (not (< ts te))))
    (match1 #t 0 0)))

;;;
;;; variable substitution
;;;

(define ($ pattern)
  ;; Shell like $ substitution of global variables in string pattern.
  ;; (define x 3)
  ;; ($subst "Hi, x = ${x}") -> "Hi, x = 3"
  ;; Variables are either ${name} or $(name) or $name
  ;; where name names a global variable.  For the third form, each
  ;; character in name must satisfy (var-char?).
  (let ((r (string-stream pattern)))
    ($subst1 (r) r '())))

(define (string-stream s)
  (let ((i 0)
	(L (string-length s)))
    (lambda ()
      (if (< i L)
	  (let ((c (string-ref s i)))
	    (set! i (+ i 1))
	    c)
	  'eof))))

(define (delimiter? c) (or (eqv? c #\{) (eqv? c #\()))

(define (end-delimiter c)
  (cond ((eqv? c #\() #\))
	((eqv? c #\{) #\})))
  
(define (var-char? x) 
  (or (char-alphabetic? x) (char-numeric? x) (eqv? x #\_)))

(define ($subst1 c r so-far-0)
  (define ($subst1$-delimited c delimiter so-far)
    (cond ((or (eqv? c delimiter) (eqv? c 'eof))
	   ($subst1 (r) r (cons 
			   (eval (string->symbol
				  (list->string (reverse so-far)))
				 this-environment)
			   so-far-0)))
	  (else ($subst1$-delimited (r) delimiter (cons c so-far)))))
  (define ($subst1$ c so-far)
    (cond
     ((eqv? c 'eof) 
      ($subst1 (r) r (cons
			    (eval (string->symbol
				   (list->string (reverse so-far)))
				  this-environment)
			    so-far-0)))
     ((var-char? c) ($subst1$ (r) (cons c so-far)))
     (else ($subst1$ (r) (cons
			    (eval (string->symbol
				   (list->string (reverse so-far)))
				  this-environment)
			    so-far-0)))))

  (cond ((eqv? c #\$)
	 (let ((c1 (r)))
	   (cond ((delimiter? c1)
		  ($subst1$-delimited (r) (end-delimiter c1) '()))
		 ((eq? c 'eof) (error "$ at end of string"))
		 (else ($subst1$ c1 '())))))
	((eqv? c 'eof) (apply string-append (reverse so-far-0)))
	(else ($subst1 (r) r (cons c so-far-0)))))

