;;;
;;; awk
;;;

(load "elf/basic.scm")
(import "gnu.regexp.RE")
(import "gnu.regexp.REMatchEnumeration")


;;; Olin Shivers at the Little Langauges Workshop, Nov, 17, 2001 at MIT.

;;; It currently uses the gnu.regexp. 1.0.8 for regular expression
;;; matching.  I can't figure out how to get it to break records so
;;; use (file-lines) which returns a BufferedReader that silk can
;;; iterate over a line at a time.

(define (file-lines file)
  ;; Convert a file into a list of lines.
  (java.io.BufferedReader.
	   (java.io.InputStreamReader.
	    (java.io.FileInputStream.
	     file))))

(define (awk-lines)
  ;; Used for testing.
  (file-lines "d:/silk4/silkweb/elf/awk.data"))

;;; A variable, re, means a regular expression.  You may need to
;;; double escape some re's to get around Scheme's syntax.  For example:
;;; "\\s+" matches one or more space characters.

;;; A field-splitter is a (lambda (record)) that splits a record into
;;; a list of fields.

(define (field-split re)
  ;; Returns a record-splitter that separates fields by matching re.
  ;; Example: To separate a record by any number of blank spaces:
  ;; (field-split \"\\s+\")
  (let ((re (RE. re)))
    (lambda (record)
      (let ((result '())
	    (start 0))
	(iterate (.getMatchEnumeration re record)
		 (lambda (m)
		   (set! result (cons
				 (.substring record start (.getStartIndex m))
				 result))
		   (set! start (.getEndIndex m))))
	(set! result (cons (.substring record start) result))
	(reverse result)))))

(define (field-match re)
  (let ((re (RE. re)))
    (lambda (record)
      (let ((result '()))
	(iterate (.getMatchEnumeration re record)
		 (lambda (m)
		   (set! result (cons
				 (.substring record
					     (.getStartIndex m)
					     (.getEndIndex   m))
				 result))))
	(reverse result)))))

(define data (map* (field-split "\\s\\s*") (awk-lines)))

(define re (RE. "\\s\\s*"))

(define re (RE. "^foo\\.(.*)\\.java$"))
(.getMatch re "foo.bar.java")

(define re (RE. ".*$"))

(define record "-rw-r--r--   1 544      everyone     4067 Sep  4 12:25 GCMonitor.scm")

(define nums ((field-match "-?[0-9]+") record))

("ABC<>DEF<>GHI<>JKL" 