// Copyright 2001, Jonathan Bachrach.  See file TERMS.

import java.io.*;
import java.util.*;

public class syntaxSwitchSExpander implements SyntaxExpander {
    private static String[] continuationWords = {};
    public String[] getContinuationWords () { return continuationWords; }

    public boolean isAux () { return false; }

    public Expansion expand (Fragment fragments) throws SyntaxMatchFailure {
	// case #{ syntaxSwitch (?:expression) { 
        //           case ?pattern:expression : return ?template:expression ; 
        //         } }:
	//   return #{ ?(pattern.compilePattern) ?(template.compileTemplate) }

	FragmentList elements = fragments.getInsideFragments().getFragmentListTail();
	LooseFragmentParser.trace("EXPANDING " + elements);
	try {
	    Expansion expression = null ;
	    FragmentList fs0 = elements;
	    SplitList tmp1 = fs0 . matchParens () ;
	    FragmentList ns2 = tmp1 . getInside () ;
	    SplitList tmp5 = ns2 . matchConstraint ("expression") ;
	    expression = new Expansion (tmp5 . getBefore ()) ;
	    IdentifierFragment expressionVar = IdentifierFragment.genSym("exp");
	    Expansion auxPrologue = null;
	    IdentifierFragment nextVar = null;
	    if (isAux()) {
		nextVar          = IdentifierFragment.genSym("nxt");
		FragmentList ns3 = tmp5.getAfter();
		FragmentList ns4 = ns3 . matchComma();
		SplitList tmp6   = ns4 . matchConstraint("expression");
		Expansion next   = new Expansion (tmp6 . getBefore ());
		tmp6 . getAfter() . matchEmpty();
		auxPrologue      =
		    Template.processTemplate
		    (FragmentList.fnil
		     .fpush(Template.substituteIdentifier("FragmentListRef"))
		     .fpush(nextVar)
		     .fpush(Template.substituteIdentifier("="))
		     .fpush(next)
		     .fpush(Template.substituteSemicolon())
		     .freverse());
	    } else {
		tmp5 . getAfter() . matchEmpty();
		auxPrologue = Template.processTemplate(FragmentList.fnil);
	    }
	    FragmentList fs4 = tmp1 . getAfter () ;
	    List rules = processSyntaxSwitchBlock(fs4);
	    Expansion ruleCode = codeGenSyntaxSwitch(rules, expressionVar, nextVar);

	    if (isAux())
		return Template.processTemplate
		    (FragmentList.fnil
		     .fpush(Template.substituteIdentifier("FragmentList"))
		     .fpush(expressionVar)
		     .fpush(Template.substituteIdentifier("="))
		     .fpush(expression)
		     .fpush(Template.substituteSemicolon())
		     .fpush(auxPrologue)
		     .fpush(ruleCode)
		     .freverse());
	    else 
		return Template.processTemplate
		    (FragmentList.fnil
		     .fpush(Template.substituteIdentifier("Fragment"))
		     .fpush(expressionVar)
		     .fpush(Template.substituteIdentifier("="))
		     .fpush(expression)
		     .fpush(Template.substituteSemicolon())
		     .fpush(auxPrologue)
		     .fpush(ruleCode)
		     .freverse());

	} catch (Exception e) {
	    LooseFragmentParser.debugOut("Failed to match syntaxSwitch " + e, elements);
	    e.printStackTrace();   // so we can get stack trace		
	    elements.pprint(Fragment.out, 0);
	    return new Expansion(FragmentList.fnil);
	}
    }

    public FragmentList processSyntaxSwitchBlockPattern
	(FragmentList fragments, SyntaxRule rule) throws SyntaxMatchFailure {
	LooseFragmentParser.debugOut("processSyntaxSwitchBlockPattern: ", fragments);
	try {
	    FragmentList next1  = fragments . matchName ("case") ;
	    LooseFragmentParser.trace("  case");
	    SplitList tmpvals   = next1 . matchTemplate () ;
	    LooseFragmentParser.trace("  template");
	    Template pattern    = (Template)tmpvals . getBefore () . getFragmentHead () ;
	    FragmentList next2  = tmpvals . getAfter () ;
	    FragmentList next3  = next2 . matchName (":") ;
	    LooseFragmentParser.trace("  colon");
	    rule.addPattern(pattern);
	    return next3;
	} catch (SyntaxMatchFailure e) {
	    LooseFragmentParser.debugOut("  pattern failed at: ", fragments);
	    throw e;
	    /* TODO: add default pattern
	    FragmentList next1  = fragments . matchName ("default") ;
	    FragmentList next3 = next2 . matchName (":") ;
	    rule.addPattern(pattern);
	    return next3;
	    */
	}
    }

    public FragmentList processSyntaxSwitchBlockPatterns
	(FragmentList fragments, SyntaxRule rule) throws SyntaxMatchFailure {
	LooseFragmentParser.debugOut("processSyntaxSwitchBlockPatterns: ", fragments);
	FragmentList next = fragments;
	while (true) {
	    try {
		next = processSyntaxSwitchBlockPattern(next, rule);
	    } catch (SyntaxMatchFailure e) {
		LooseFragmentParser.debugOut("  patterns failed at: ", next);
		break;
	    }
	}
	return next;
    }
    public int findNextCase (FragmentList fragments) {
	FragmentList ptr = fragments;
	for (int i = 0; ptr != FragmentList.fnil; i++) {
	    Fragment f = ptr.getFragmentHead();
	    if (f instanceof IdentifierFragment) {
		String name = ((IdentifierFragment)f).getName();
		if (name.equals("case") || name.equals("default"))
		    return i;
	    }
	    ptr = ptr.getFragmentListTail();
	}
	return fragments.size();
    }
    public FragmentList processSyntaxSwitchBlockStatements 
	(FragmentList fragments, SyntaxRule rule) throws SyntaxMatchFailure {
	LooseFragmentParser.debugOut("processSyntaxSwitchBlockStatements: ", fragments);
	FragmentList next       = fragments;
	// while (true) {
	    try {
		int lim                = findNextCase(next);
		SplitList tmp          = next.matchConstraint("statement", lim);
		FragmentList statement = tmp.getBefore();
		LooseFragmentParser.trace("  Before " + statement);
		rule.addStatement(statement);
		next = tmp.getAfter();
		LooseFragmentParser.trace("  After   " + next);
	    } catch (SyntaxMatchFailure e) {
		LooseFragmentParser.debugOut("  statement failed at: ", next);
		if (rule.getStatements().isEmpty()) {
		    LooseFragmentParser.trace("No Statements found");
		    throw e;
		} else 
		    return next;
	    }
       // }
       return next;
    }
    public FragmentList processSyntaxSwitchBlockRule 
	(FragmentList fragments, SyntaxRule rule) throws SyntaxMatchFailure {
	LooseFragmentParser.debugOut("processSyntaxSwitchBlockRule: ", fragments);
	FragmentList statements
	    = processSyntaxSwitchBlockPatterns(fragments, rule);
	FragmentList next
	    = processSyntaxSwitchBlockStatements(statements, rule);
	return next;
    }
    public List processSyntaxSwitchBlock 
	(FragmentList fragments) throws SyntaxMatchFailure {
	LooseFragmentParser.debugOut("processSyntaxSwitchBlock: ", fragments);
	// syntaxSwitchBlock:
	//   #{ { ?syntaxSwitchBlockStatementGroups } }
	// syntaxSwitchBlockStatementGroups:
	//   #{ ?syntaxSwitchBlockStatementGroup ?syntaxSwitchBlockStatementGroups }
	//   #{ ?syntaxSwitchBlockStatementGroup }
	SplitList tmp                = fragments . matchBraces () ;
	FragmentList statementGroups = tmp . getInside () ;
	List rules                   = List.nil;
	while (!statementGroups.isEmpty()) {
	    SyntaxRule rule = new SyntaxRule();
	    statementGroups = processSyntaxSwitchBlockRule(statementGroups, rule);
	    rules           = rules.push(rule);
	}
	return rules.reverse();
    }

    public Expansion codeGenSyntaxSwitch (List rules, Fragment expression, IdentifierFragment next) {
	// try {
	//   ?pattern-code-1
	//   ?matched-code-1
	// } catch (SyntaxMatchFailure e) {
	//   try {
	//     ?pattern-code-2
	//     ?matched-code-2
	//   } catch (SyntaxMatchFailure e) {
	//     throw new SyntaxMatchFailure(...);
	//   }
	// }

	if (rules.isEmpty()) {
	    return Template.processTemplate
		(FragmentList.fnil
		 .fpush(Template.substituteIdentifier("throw"))
		 .fpush(Template.substituteIdentifier("new"))
		 .fpush(Template.substituteIdentifier("SyntaxMatchFailure"))
		 .fpush(Template.processParens(FragmentList.fnil))
		 .fpush(Template.substituteSemicolon())
		 .freverse());
	} else {
	    SyntaxRule rule                 = (SyntaxRule)rules.getHead();
	    Template pattern                = (Template)rule.getPatterns().getHead();
	    FragmentList statements         = rule.getStatements();
	    FragmentList expandedStatements = statements.expandTemplates();
	    return Template.processTemplate
		(FragmentList.fnil
		 .fpush(Template.substituteIdentifier("try"))
		 .fpush(Template.processBraces
			(FragmentList.fnil
			 .fpush(pattern.compilePattern(expression, next))
			 .fpush(new Expansion(expandedStatements))
			 .freverse()))
		 .fpush(Template.substituteIdentifier("catch"))
		 .fpush(Template.processParens
			(FragmentList.fnil
			 .fpush(Template.substituteIdentifier("SyntaxMatchFailure"))
			 .fpush(IdentifierFragment.genSym("e"))
			 .freverse()))
		 .fpush(Template.processBraces
			(FragmentList.fnil
			 .fpush(codeGenSyntaxSwitch(rules.getTail(), expression, next))))
		 .freverse());
	}
    }

}
